/*
*         Copyright (c), NXP Semiconductors Bangalore / India
*
*                     (C)NXP Semiconductors
*       All rights are reserved. Reproduction in whole or in part is
*      prohibited without the written consent of the copyright owner.
*  NXP reserves the right to make changes without notice at any time.
* NXP makes no warranty, expressed, implied or statutory, including but
* not limited to any implied warranty of merchantability or fitness for any
*particular purpose, or that the use will not infringe any third party patent,
* copyright or trademark. NXP must not be liable for any loss or damage
*                          arising from its use.
*/

/** \file
* Generic phDriver Component of Reader Library Framework.
* $Author$
* $Revision$
* $Date$
*
* History:
*  RS: Generated 24. Jan 2017
*
*/

/* *****************************************************************************************************************
 * Includes
 * ***************************************************************************************************************** */

#include "BoardSelection.h"
#include "phDriver.h"
#include <Board_LPC11U37.h>
#include "board.h"

/* *****************************************************************************************************************
 * Internal Definitions
 * ***************************************************************************************************************** */
#define KINETIS_TIMER_MAX_32BIT      0xFFFFFFFFU
#define BOARD_XTAL0_CLK_HZ 32000000U

/* *****************************************************************************************************************
 * Type Definitions
 * ***************************************************************************************************************** */


/* *****************************************************************************************************************
 * Global and Static Variables
 * Total Size: NNNbytes
 * ***************************************************************************************************************** */

static pphDriver_TimerCallBck_t pTimer0_CallBack;
static volatile uint8_t dwTimerExp;
static void phDriver_TimerIsrCallBack(void);

/* *****************************************************************************************************************
 * Public Functions
 * ***************************************************************************************************************** */

phStatus_t phDriver_TimerStart(phDriver_Timer_Unit_t eTimerUnit, uint32_t dwTimePeriod, pphDriver_TimerCallBck_t pTimerCallBack)
{
	uint64_t          qwTimerCnt;
	uint32_t          dwTimerFreq;
	dwTimerFreq = Chip_Clock_GetIntOscRate();
	/* Timer count = (delay * freq)/Units. */
	qwTimerCnt = dwTimerFreq;
	qwTimerCnt = (qwTimerCnt / eTimerUnit);
	qwTimerCnt = (dwTimePeriod * qwTimerCnt);

	/* 32-bit timers. */
	if(qwTimerCnt > (uint64_t)KINETIS_TIMER_MAX_32BIT)
	{
		return PH_DRIVER_ERROR | PH_COMP_DRIVER;
	}

	if(pTimerCallBack == NULL)
	{    /* Timer Start is blocking call. */
		dwTimerExp = 0;
		pTimer0_CallBack = phDriver_TimerIsrCallBack;
	}
	else
	{   /* Call the Timer callback. */
		pTimer0_CallBack = pTimerCallBack;
	}

	/* Enable timer 1 clock */
	Chip_TIMER_Init(LPC_TIMER32_0);

	Chip_TIMER_Reset(LPC_TIMER32_0);
	Chip_TIMER_MatchEnableInt(LPC_TIMER32_0, 1);
	Chip_TIMER_SetMatch(LPC_TIMER32_0, 1, qwTimerCnt);
	Chip_TIMER_ResetOnMatchEnable(LPC_TIMER32_0, 1);
	Chip_TIMER_Enable(LPC_TIMER32_0);

	/* Enable timer interrupt */
	NVIC_ClearPendingIRQ(TIMER_32_0_IRQn);
	NVIC_EnableIRQ(TIMER_32_0_IRQn);

	if(pTimerCallBack == NULL)
	{
		/* Block until timer expires. */
		while(!dwTimerExp);
	}

    return PH_DRIVER_SUCCESS;
}


phStatus_t phDriver_TimerStop(void)
{
	Chip_TIMER_Reset(LPC_TIMER32_0);
	Chip_TIMER_Disable(LPC_TIMER32_0);
	/* Disable timer interrupt */
	NVIC_DisableIRQ(TIMER_32_0_IRQn);

    return PH_DRIVER_SUCCESS;
}

phStatus_t phDriver_PinConfig(uint32_t dwPinNumber, phDriver_Pin_Func_t ePinFunc, uint32_t pinConfig, uint8_t pinIntGroup, phDriver_Interrupt_Config_t intConfig)
{
	uint8_t bPinNum;
	uint8_t bPortGpio;

	/* Extract the Pin, Gpio, Port details from dwPinNumber */
	bPinNum = (uint8_t)(dwPinNumber & 0xFF);
	bPortGpio = (uint8_t)((dwPinNumber & 0xFF00)>>8);

	if(ePinFunc == PH_DRIVER_PINFUNC_BIDIR)
	{
		return PH_DRIVER_ERROR | PH_COMP_DRIVER;
	}

	/* Configure ED pin as input pin */
	Chip_GPIO_SetPinDIRInput(LPC_GPIO, bPortGpio, bPinNum);
	/* Configure ED pin with pullup */
	Chip_IOCON_PinMuxSet(LPC_IOCON, bPortGpio, bPinNum, pinConfig);

	if(ePinFunc == PH_DRIVER_PINFUNC_INTERRUPT)
	{
		/* Enable PININT clock */
		Chip_Clock_EnablePeriphClock(SYSCTL_CLOCK_PINT);
		/* Configure interrupt channel for the GPIO pin in SysCon block */
		Chip_SYSCTL_SetPinInterrupt(pinIntGroup, bPortGpio, bPinNum);

		switch(intConfig)
		{
		case PH_DRIVER_INTERRUPT_LEVELONE:
		case PH_DRIVER_INTERRUPT_RISINGEDGE:
			Chip_PININT_SetPinModeEdge(LPC_PININT, PININTCH(pinIntGroup));
			Chip_PININT_EnableIntHigh(LPC_PININT, PININTCH(pinIntGroup));
			break;
		case PH_DRIVER_INTERRUPT_LEVELZERO:
		case PH_DRIVER_INTERRUPT_FALLINGEDGE:
			Chip_PININT_SetPinModeEdge(LPC_PININT, PININTCH(pinIntGroup));
			Chip_PININT_EnableIntLow(LPC_PININT, PININTCH(pinIntGroup));
			break;
		case PH_DRIVER_INTERRUPT_EITHEREDGE:
			Chip_PININT_SetPinModeEdge(LPC_PININT, PININTCH(pinIntGroup));
			Chip_PININT_EnableIntHigh(LPC_PININT, PININTCH(pinIntGroup));
			Chip_PININT_EnableIntLow(LPC_PININT, PININTCH(pinIntGroup));
			break;
		}

		NVIC_SetPriority(PININT_NVIC_NAME, 8);
		/* Enable interrupt in the NVIC */
		NVIC_EnableIRQ(PININT_NVIC_NAME);
	}

    return PH_DRIVER_SUCCESS;
}


void phDriver_PinClearIntStatus(uint8_t pinIntGroup)
{
	Chip_PININT_ClearIntStatus(LPC_PININT, PININTCH(pinIntGroup));
}

/* Timer IRQ Handler */
void TIMER32_0_IRQHandler(void)
{
	if (Chip_TIMER_MatchPending(LPC_TIMER32_0, 1)) {
		Chip_TIMER_ClearMatch(LPC_TIMER32_0, 1);
	}

	Chip_TIMER_Disable(LPC_TIMER32_0);
	/* Disable timer interrupt */
	NVIC_DisableIRQ(TIMER_32_0_IRQn);

	pTimer0_CallBack();
}

static void phDriver_TimerIsrCallBack(void)
{
    dwTimerExp = 1;
}


