/*
 * Copyright (c) 2016, NXP Semiconductor
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#ifndef _NTAG_AUTH_H_
#define _NTAG_AUTH_H_
/** @file ntag_auth.h
 * \brief Public interface to access standby functionality of NTAG over I2C.
 */

/** \defgroup NTAG_AUTH
*
* \brief This component implements API responsible for Plain Password security over I2C
* @{
*/

/***********************************************************************/
/* INCLUDES                                                            */
/***********************************************************************/
#include "global_types.h"
#include "ntag_defines.h"

/***********************************************************************/
/* DEFINES                                                             */
/***********************************************************************/

/***********************************************************************/
/* TYPES                                                               */
/***********************************************************************/


/**
 * \brief Operation to be performed.
 */
typedef enum {
    NTAG_READ = 1,              /**< Read operation */
    NTAG_WRITE = 2,             /**< Write operation */
    NTAG_RESTRICTEDREAD = 3,    /**<  Restricted Read operation */
    NTAG_RESTRICTEDWRITE = 4    /**<  Restricted Write operation */
} NTAG_OPERATION_T;


/**
 * \brief Security Parameter to be modified
 */
typedef enum {
    NTAG_KCH = 1,                                          /**< Key Header Configuration */
    NTAG_PROTECTIONPOINTER = 2,                            /**< Protection Pointer */
    NTAG_PROTECTIONPOINTER_CONFIGURATION = 3,              /**< Protection Pointer Configuration */
    NTAG_RESTRICTEDAREA = 4                                /**<  Restricted Area */
} NTAG_SECURITY_PARAMETER_T;


/**
 * \brief Protecion Pointer Configuration combinations
 */
typedef enum {
       ReadLow = 0x01,           /* Page L is read protected */
       WriteLow = 0x02,          /* Page L is write protected */
       ReadHigh = 0x10,          /* Page H is read protected */
       WriteHigh = 0x20,         /* Page H is write protected */
       ReadLow_WriteLow = 0x03,  /* Page L is read and write protected */
       ReadLow_WriteHigh = 0x21, /* Page L is read and page H is write protected */
       ReadHigh_WriteLow = 0x12, /* Page H is read and page L is write protected */
       ReadHigh_WriteHigh = 0x30,/* Page H is read and page H is write protected */
       ReadLow_ReadHigh = 0x11,   /* Page L is read and page H is read protected */
       WriteLow_WriteHigh = 0x22  /* Page L is write and page H is write protected */

} NTAG_PROTECTION_POINTER_CONFIGURATION_T;
/***********************************************************************/
/* GLOBAL VARIABLES                                                    */
/***********************************************************************/

/***********************************************************************/
/* GLOBAL FUNCTION PROTOTYPES                                          */
/***********************************************************************/

/**
 * \brief Writes the key of the indicated operation
 *
 * This function writes the key of the specified memory operation \ref NTAG_OPERATION_T
 *
* \param   ntag            handle to identify the NTAG device instance
 * \param   operation      Memory Operation for which the key to be written
 * \param   byte           Buffer oF 4 Bytes which contains the password for the corresponding operation
 *
 * \return          true on Failure
 */
BOOL NTAG_I2C_WRITE_KEY(NTAG_HANDLE_T ntag, NTAG_OPERATION_T operation, uint8_t* byte);


/**
 * \brief Reads the key of the indicated operation
 *
 * This function reads and returns the key of the specified operation \ref NTAG_OPERATION_T
 *
 * \param   ntag        handle to identify the NTAG device instance
 * \param   operation   Memory operation whose key has to be read
 * \param   byte         Buffer oF 4 Bytes which returns the password for the corresponding memory operation
 *
 * \return          true on Failure
 */
BOOL NTAG_I2C_READ_KEY(NTAG_HANDLE_T ntag, NTAG_OPERATION_T operation, uint8_t* byte);


/**
 * \brief Performs the authentication operation
 *
 *  This functions performs the authentication on the selected operation.
 *  To perform the authentication the operation relies on the key set by the
 *  NTAG_I2C_WRITE_KEY
 *
 * \param   ntag        handle to identify the NTAG device instance
 * \param   operation   Memory operation for which to perform authentication \ref NTAG_OPERATION_T
 * \param   byte        Buffer oF 4 Bytes which contains the password for the corresponding operation
 *
 * \return          true on Failure
 */
BOOL NTAG_I2C_AUTHENTICATION(NTAG_HANDLE_T ntag, NTAG_OPERATION_T operation, uint8_t* byte);


/*
 * \brief To set the Specified Security Parameter over I2C \ref NTAG_SECURITY_PARAMETER_T
 *
 * \param   ntag        handle to identify the NTAG device instance
 * \param   Parameter   Security Parameter \ref NTAG_SECURITY_PARAMETER_T to be set
 * \param   value       unsigned short value to be written to the security parameter,
 *                      only lsb 8-bits to be used for NTAG_KCH, NTAG_PROTECTIONPOINTER and NTAG_PROTECTIONPOINTER_CONFIGURATION
 *                      and entire 16-bits for NTAG_RESTRICTEDAREA
 *
 * \return          true on Failure
 *
 */
BOOL NTAG_I2C_SET_SecurityParameter(NTAG_HANDLE_T ntag, NTAG_SECURITY_PARAMETER_T Parameter, uint16_t value);

/*
 * \brief To Get the Specified Security Parameter over I2C \ref NTAG_SECURITY_PARAMETER_T
 *
 * \param   ntag        handle to identify the NTAG device instance
 * \param   Parameter   Security Parameter \ref NTAG_SECURITY_PARAMETER_T to retrieve
 * \param   value       16-bit value as retrieved from the security parameter. Relevent LSB 8-bit will be returned for
 *                      NTAG_KCH, NTAG_PROTECTIONPOINTER and NTAG_PROTECTIONPOINTER_CONFIGURATION and entire
 *                      16-bits for NTAG_RESTRICTEDAREA
 *
 * \return    True on Failure
 */
BOOL NTAG_I2C_GET_SecurityParameter(NTAG_HANDLE_T ntag, NTAG_SECURITY_PARAMETER_T Parameter, uint16_t* value);

/*
 * \brief To set the Authentication Limit
 * \param    ntag        handle to identify the NTAG device interrupt
 * \param    value       16-bit value to be written to the auth counter, the MSb is
 *                       ignored as it is to set to by \ref NTAG_I2C_AuthCounter_OnOff
 *
 * \return    True on Failure
 */
BOOL NTAG_I2C_Set_AuthLimit(NTAG_HANDLE_T ntag, uint16_t value);

/*
 * \brief To Read the Authentication Limit
 * \param    ntag        handle to identify the NTAG device interrupt
 * \param    value       16-bit return value of the auth limit, the return value will indicate with its MSb
 *                       whether counter is on or off
 *
 * \return    True on Failure
 */
BOOL NTAG_I2C_Read_AuthLimit(NTAG_HANDLE_T ntag, uint16_t* value);
/*
 * \brief To enable\disable the Auth Counter
 * \param    ntag        handle to identify the NTAG device interrupt
 * \param    On          the bool value to enable\disable the Auth Counter
 *
 * \return    True on Failure
 */
BOOL NTAG_I2C_AuthCounter_OnOff(NTAG_HANDLE_T ntag, Bool on);



#endif /* _NTAG_AUTH_H_ */
