/*
 * Copyright (c) 2016, NXP Semiconductor
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#ifndef _NTAG_BRIDGE_H_
#define _NTAG_BRIDGE_H_
/** @file ntag_bridge.h
 * \brief Public interface for using an NTAG I2C tag as I2C<->NFC bridge.
 */

/** \defgroup NTAG_BRIDGE
*
* \brief This component implements API responsible for Bridging the transaction between NTAG and Microcontroller
* @{
*/

/***********************************************************************/
/* INCLUDES                                                            */
/***********************************************************************/
#include "global_types.h"
#include "ntag_driver.h"

/***********************************************************************/
/* DEFINES                                                             */
/***********************************************************************/
#define NTAG_TAG_STATUS_PT_TRANSFER_DIR 1
/***********************************************************************/
/* TYPES                                                               */
/***********************************************************************/

/**
 * \brief The function to set the event detection Function
 */
typedef enum
{
    CLEAR_ED_PIN_0000b = 0,                                   /**< Clears ED pin. ED = OFF */
    RF_SWITCHED_ON_OR_OFF_0001b = 1,                          /**< RF Field Present or NOT Present */
    PWM0_Signal_0010b = 2,                                    /**< PWM0 signal */
    PASS_THROUGH_MODE_I2C_TO_RF_0011b = 3,                    /**< PThru I2C -> RF Last byte Read from RF\written by I2C */
    PASS_THROUGH_MODE_RF_TO_I2C_0100b = 4,                    /**< PThru RF -> I2C Last byte written by RF\Read from I2C  */
    ARBITER_LOCKED_TO_RF_0101b = 5,                           /**< RF IF LOCKED\NOT */
    NDEFDATA_ZERO_OR_NONZERO_0110b = 6,                       /**< Ndef Length Byte is Zero or Non Zero during write command */
    STANDBY_MODE_0111b = 7,                                   /**< StandBy Mode Active\Inactive */
    WRITE_COMMAND_ONGOING_1000b = 8,                          /**< Start\End of Write command received */
    READ_COMMAND_ONGOING_1001b = 9,                           /**< Start\End of Read command received */
    SOF_OR_EOF_DETECTED_1010b = 10,                           /**< SOF\ end of response detected */
    READ_FROM_NDEF_DATA_BLOCK_OR_FIELD_OFF_1011b = 11,        /**< NDEF DATA BLOCK read or field off */
    WRITTEN_TO_NDEF_DATA_BLOCK_OR_FIELD_OFF_1100b = 12,       /**< NDEF DATA BLOCK write or field off */
    SOFTWARE_INITIATED_INTERRUPT_1101b  = 13,                 /**< Software Initiated Interrupt */
    ACTIVE_RF_DETECT_1110b = 14,                              /**< Active RF Detect */
    ACTIVE_RF_OK_1111b = 15                                   /**< Active RF Detect */
} NTAG_ED_FUNCTIONS_T;

/**
 *  Transfer Direction of Pass Through Mode.
 */
typedef enum
{
    RF_TO_I2C = NTAG_TAG_STATUS_PT_TRANSFER_DIR,  /**< Transfer direction is RF to I2C */
    I2C_TO_RF = 0                                 /**< Transfer direction is I2C to RF */
} NTAG_TRANSFER_DIR_T;

/**
 * \brief NTAG Event set
 */
typedef enum
{
    NTAG_EVENT_ED_PIN_HI_INTERRUPT,                   /**< Configuring for HI Interrupt */
    NTAG_EVENT_ED_PIN_LO_INTERRUPT,                   /**< Configuring for LOW Interrupt */
    NTAG_EVENT_RF_FIELD_PRESENT_INTERRUPT,            /**< RF Field Present */
    NTAG_EVENT_RF_FIELD_NOT_PRESENT_INTERRUPT,        /**< RF Field Not Present */
    NTAG_EVENT_PWM0_ON,                               /**< PWM0 is on */
    NTAG_EVENT_PWM0_OFF,                              /**< PWM0 is off */
    NTAG_EVENT_LAST_BYTE_READ_FROM_RF,                /**< Last Byte of Data Read from RF */
    NTAG_EVENT_LAST_BYTE_WRITTEN_BY_I2C,              /**< Last Byte of Data written BY I2C */
    NTAG_EVENT_LAST_BYTE_WRITTEN_BY_RF,               /**< Last Byte of Data written by RF */
    NTAG_EVENT_LAST_BYTE_READ_FROM_I2C,               /**< Last Byte of Data Read from I2C */
    NTAG_EVENT_ARBITER_LOCKED_BY_RF,                  /**< Arbiter is Locked to RF */
    NTAG_EVENT_ARBITER_NOT_LOCKED_BY_RF,              /**< Arbiter is released from RF */
    NTAG_EVENT_NDEF_LENGTH_BYTE_NONZERO,              /**< NDEF Length Byte is Non zero during write command */
    NTAG_EVENT_NDEF_LENGTH_BYTE_ZERO,                 /**< NDEF Length Byte is zero during write command */
    NTAG_EVENT_STANDBY_INACTIVE,                      /**< IC is not in standby mode */
    NTAG_EVENT_STANDBY_ACTIVE,                        /**< IC is in standby mode */
    NTAG_EVENT_START_OF_WRITE,                        /**< Start of Write Command Received */
    NTAG_EVENT_END_OF_WRITE,                          /**< End of Write Command Received */
    NTAG_EVENT_START_OF_READ,                         /**< Start of Read Command Received */
    NTAG_EVENT_END_OF_READ,                           /**< End of Read Command Received */
    NTAG_EVENT_SOF_DETECTED,                          /**< Start of Frame Detected */
    NTAG_EVENT_EOF_DETECTED,                          /**< End of Response Detected */
    NTAG_EVENT_NDEF_DATA_BLOCK_READ,                  /**< NDEF DATA BLOCK read */
    NTAG_EVENT_NDEF_DATA_BLOCK_WRITTEN,               /**< NDEF DATA BLOCK written */
    NTAG_EVENT_SOFTWARE_INTERRUPT,                    /**< Software Initiated Interrupt */
    NTAG_EVENT_ACTIVE_RF_DETECT_ON,                   /**< Active RF Detected */
    NTAG_EVENT_ACTIVE_RF_DETECT_OFF,                  /**< Active RF Not Detected */
    NTAG_EVENT_ACTIVE_RF_OK,                          /**< Active RF OK */
    NTAG_EVENT_ACTIVE_RF_NOT_OK                      /**< Active RF is not OK */
} NTAG_EVENT_T;

/**
 * \brief NTAG Section set
 */
typedef enum
{
    NTAG_SECTION_0,                   /**< RF Security and Crypto Configuration */
    NTAG_SECTION_1,                   /**< I2C Security configuration */
    NTAG_SECTION_2,                   /**<TAG_CONFIG */
    NTAG_SECTION_3,                   /**< Tag Configuration Section*/
    NTAG_SECTION_4,                   /**< AFI, DSFID and PP */
    NTAG_SECTION_5,                   /**< RF Lock Block Section */
    NTAG_SECTION_6,                   /**< I2C Lock Block Section */
    NTAG_SECTION_7                   /**< SRAM NDEF DATA */
} NTAG_SECTION_T;


/**
 * \brief NTAG Access Condition
 */
typedef enum
{
    RF_READABLE,                   /**< Readable over RF */
    RF_WRITABLE,                   /**< Writable over RF */
    I2C_READABLE,                  /**< Readable over I2C */
    I2C_WRITABLE,                  /**< Writable over I2C */
} NTAG_ACCESS_T;


/**
 * \brief NTAG Lock Condition
 */
typedef enum
{
    RF_NOT_READABLE,               /**< Not readable over RF */
    RF_NOT_WRITABLE,               /**< Not Writable over RF */
    I2C_NOT_READABLE,              /**< Not Readable over I2C */
    I2C_NOT_WRITABLE,              /**< Not Writable over I2C */
} NTAG_LOCK_T;

/**
 * \brief NTAG Section LOCK set
 */
typedef enum
{
    RF_SECTION_LOCK0,                   /**< RF SECTION for writing rights */
    RF_SECTION_LOCK1,                   /**< RF SECTION for reading rights */
    I2C_SECTION_LOCK0,                  /**< I2C SECTION for writing rights */
    I2C_SECTION_LOCK1                   /**< I2C SECTION for reading rights */
} NTAG_SECTION_LOCK_T;


/*
 * \brief NTAG INTERFACE whose blocks to be locked
 */
typedef enum
{
    NTAG_RF_LOCK,                  /* RF INTERFACE to be locked */
    NTAG_I2C_LOCK                  /* I2C Interface to be locked */
} NTAG_INTERFACE_T;

/***********************************************************************/
/* GLOBAL VARIABLES                                                    */
/***********************************************************************/

/***********************************************************************/
/* GLOBAL FUNCTION PROTOTYPES                                          */
/***********************************************************************/

/**
 * \brief wait for selected event
 *
 *	This functions waits until the selected event occurs or the timeout value is
 *	reached. See \ref NTAG_EVENT_T for possible events to be waited on.
 *	If you want to use the Interrupted Events set the FD function accordingly
 *	before calling this function. Notice that some Interrupted Events are
 *	indistinguishable, so it will trigger at a false event.
 *
 * \param	ntag      	handle to identify the NTAG device instance
 * \param	event     	event to be waited for
 * \param	timeout_ms	timeout value in ms
 * \param   set_fd_pin_function   when using an INTERRUPT EVENT
 * 								  if true ED_ON/ED_OFF is set to the according funtion priore to waiting for the event.
 * 								  if false nothing is done, so you should set the FD Pin function prior to calling this.
 * 								  when using an POLLING EVENT this param is ignored
 *
 * \return			true on timeout
 */
BOOL NTAG_WaitForEvent(NTAG_HANDLE_T ntag, NTAG_EVENT_T event, uint32_t timeout_ms, BOOL set_fd_pin_function);


/**
 * \brief To Clear the ED Interrupt, ED Pin is now high
 *
 * \param   ntag    handle to identify the NTAG device instance
 *
 * \return  error code as defined in ntag_defines.h
 */
BOOL NTAG_ClearED(NTAG_HANDLE_T ntag);

/**
 * \brief enable/disable the I2C SOFT RESET
 * Soft Reset releases the SCL/SDA to default state
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	on		true enables the I2C SOFT RST, false disable the I2C SOFT RST
 *
 * \return	error code as defined in ntag_defines.h
 */
BOOL NTAG_SetI2CRstOnOff(NTAG_HANDLE_T ntag, BOOL on);

/**
 * \brief Set the Value of the Section Lock to the value as provided by the user
 *  Caution: The values are one-time programmable and cannot be overwritten,
 *  means if any bit is set to 1, it cant be reset
 *
 * \param   ntag             handle to identify the NTAG device instance
 * \param   SectionLock      Section Lock Register \ref NTAG_SECTION_LOCK_T whose value to be determined
 * \param   value            The value to be written to section Lock
 *
 * \return  error code as defined in ntag_defines.h
 */
BOOL NTAG_SetI2C_SectionLock(NTAG_HANDLE_T ntag, NTAG_SECTION_LOCK_T SectionLock, uint8_t value);

/**
 * \brief Get the Value of the Section Lock
 *
 * \param   ntag             handle to identify the NTAG device instance
 * \param   SectionLock      Section Lock Register \ref NTAG_SECTION_LOCK_T whose value to be determined
 * \param   value            Value is used to store the value returned as the section LOCK
 *
 * \return  error code as defined in ntag_defines.h
 */
BOOL NTAG_GetI2C_SectionLock(NTAG_HANDLE_T ntag, NTAG_SECTION_LOCK_T SectionLock, uint8_t* value);

/**
 * \brief To Set the Access condition of the specified section of memory
 *  Caution: The values are one-time programmable and cannot be overwritten,
 *  means if any bit is set to 1, it cant be reset
 *
 * \param   ntag             handle to identify the NTAG device instance
 * \param   Section          Section \ref NTAG_SECTION_T of the memory whose Access condition is to be set
 * \param   Lock            The Lock \ref NTAG_LOCK_T condition to be set
 *
 * \return  error code as defined in ntag_defines.h
 */
BOOL NTAG_SetI2C_SectionConfiguration(NTAG_HANDLE_T ntag, NTAG_SECTION_T Section, NTAG_LOCK_T Lock);

/**
 * \brief To Get the Access condition of the specified section of memory
 *
 * \param   ntag             handle to identify the NTAG device instance
 * \param   Section          Section \ref NTAG_SECTION_T of the memory whose Access condition to get
 * \param   Access           Access Condition \ref NTAG_ACCESS_T we want to be checked
 * \param   on               whether bit is set or reset
 *
 * \return  error code as defined in ntag_defines.h
 */
BOOL NTAG_GetI2C_SectionConfiguration(NTAG_HANDLE_T ntag, NTAG_SECTION_T Section, NTAG_ACCESS_T Access, BOOL* on);

/**
 * \brief To Set the Access condition of the specified Block Memory
 *  Caution: The values are one-time programmable and cannot be overwritten,
 *  means if any bit is set to 1, it cant be reset
 *
 * \param   ntag             handle to identify the NTAG device instance
 * \param   Interface        Interface \ref NTAG_INTERFACE_T for which block is to be locked
 * \param   BlockNo           Block Number to Block. RF Interface: RF_LOCK_BLOCK0 – bit0 – will lock user blocks 0.
 *                            I2C Interface: I2C_LOCK_BLOCK0 – bit0 – will lock user blocks 0,1,2,3, means block can be locked
 *                            only in set of 4 and passing the block number lying within the subset of blocks will lock all the 4 blocks
 *                            e.g. asking to lock block 18 over I2C will lock block 16,17,18 and 19.
 *
 * \return  error code as defined in ntag_defines.h
 */
BOOL NTAG_SetI2C_LockBlock(NTAG_HANDLE_T ntag, NTAG_INTERFACE_T Interface, uint16_t BlockNo);

/**
 * \brief To Get the Access condition of the specified  blcok of Memory
 *
 * \param   ntag             handle to identify the NTAG device instance
 * \param   Interface        Interface \ref NTAG_INTERFACE_T whose access condition to get
 * \param   BlockNo          Block Number to get the access condition.
 * \param   Locked           If on the block is locked, off the block is still unlocked over the interface
 *
 * \return  error code as defined in ntag_defines.h
 */
BOOL NTAG_GetI2C_LockBlock(NTAG_HANDLE_T ntag, NTAG_INTERFACE_T Interface, uint16_t BlockNo, Bool* Locked);

/**
 * \brief To Get the Access condition of the specified  block of Memory
 *
 * \param   ntag             handle to identify the NTAG device instance
 * \param   Enable           IF on enables the NFC Silence, If Off disables the NFC Silence
 *
 * \return  error code as defined in ntag_defines.h
 */
BOOL NTAG_SetI2C_NFCSILENCE(NTAG_HANDLE_T ntag,  Bool Enable);

/**
 * \brief Sets the I2C_LOCKED bit to 0 to free the LOCK immediately
 *
 * \param   ntag    handle to identify the NTAG device instance
 *
 * \return          true on failure*
 */
BOOL NTAG_ReleaseI2CLocked(NTAG_HANDLE_T ntag);

/**
 * \brief set the transfer direction
 *
 * If the PT direction is already the desired direction nothing is done.
 * If the Pthru is switched on, it will be switched off and back on after the direction change.
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	dir		Direction of the transfer
 *
 * \return			true on failure*
 */
BOOL NTAG_SetTransferDir(NTAG_HANDLE_T ntag, NTAG_TRANSFER_DIR_T dir);

/**
 * \brief get the transfer direction
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	dir		Direction of the transfer
 *
 * \return			true on failure*
 */
BOOL NTAG_GetTransferDir(NTAG_HANDLE_T ntag, NTAG_TRANSFER_DIR_T *dir);

/**
 * \brief set the Event Detection function
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	func	Function which should be set
 *
 * \return			true on failure*
 */
BOOL NTAG_SetEDFunction(NTAG_HANDLE_T ntag, NTAG_ED_FUNCTIONS_T func);

/**
 * \brief get the Event Detection function
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	func	Function which is set
 *
 * \return			true on failure*
 */
BOOL NTAG_GetEDFunction(NTAG_HANDLE_T ntag, NTAG_ED_FUNCTIONS_T *func);

/**
 * \brief enable/disable the PHDC Mode
 *
 * Activates the PHDC mode, that uses the NTAG's 256 Byte SRAM
 * for communication between a NFC device and the I2C bus
 *
 * \param   ntag    handle to identify the NTAG device instance
 * \param   on      true enables the PHDC , false disable the PHDC
 *
 * \return  error code as defined in ntag_defines.h
 */
BOOL NTAG_SetPHDCOnOff(NTAG_HANDLE_T ntag, BOOL on);

/**
 * \brief get the status of the PHDC Mode
 *
 * Activates the pass-through mode, that uses the NTAG's 256 Byte SRAM
 * for communication between a NFC device and the I2C bus
 *
 * \param   ntag    handle to identify the NTAG device instance
 * \param   on      true PHDC is enabled , false PHDC is disabled
 *
 * \return  error code as defined in ntag_defines.h
 */
BOOL NTAG_GetPHDCOnOff(NTAG_HANDLE_T ntag, BOOL *on);

/**
 * \brief enable/disable the I2C<->NFC bridge
 *
 * Activates the pass-through mode, that uses the NTAG's 256 Byte SRAM
 * for communication between a NFC device and the I2C bus
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	on		true enables the Pthru , false disable the Pthru
 *
 * \return	error code as defined in ntag_defines.h
 */
BOOL NTAG_SetPthruOnOff(NTAG_HANDLE_T ntag, BOOL on);

/**
 * \brief get the status of the Pthru
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	on		status of Pthru, true is on, false is off
 *
 * \return	error code as defined in ntag_defines.h
 */
BOOL NTAG_GetPthruOnOff(NTAG_HANDLE_T ntag, BOOL *on);

/**
 * \brief enable/disable SRAM projection to user memory addresses.
 * The projection will always start from the zeroth block
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	on		true enables the SRAM Mirror , false disable the SRAM Mirror
 *
 * \return			true on failure*
 */
BOOL NTAG_SetSRAMMirrorOnOff(NTAG_HANDLE_T ntag, BOOL on);

/**
 * \brief get the status of the SRAM Mirror to user memory
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	on		status of the SRAM Mirror, true is on, false is off
 *
 * \return			true on failure*
 */
BOOL NTAG_GetSRAMMirrorOnOff(NTAG_HANDLE_T ntag, BOOL *on);

/**
 * \brief change the LAST_NDEF_BLOCK
 *
 *  Change the first block of user memory when using the LAST_NDEF_BLOCK.
 *  Granularity is 4 pages(4bytes each), so block * 4 is the real page address.
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	block	block address
 * \return			true on failure*
 */
BOOL NTAG_SetLastNDEFBlock(NTAG_HANDLE_T ntag, uint16_t block);

/**
 * \brief get the LAST_NDEF_BLOCK
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	block	LAST_NDEF_BLOCK
 *
 * \return			true on failure*
 */
BOOL NTAG_GetLastNDEFBlock(NTAG_HANDLE_T ntag, uint16_t *block);

/**
 * \brief change the value of the watchdog timer
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	time	new time value of the watchdog timer, gets applied at the next POR
 * \return			true on failure*
 */
BOOL NTAG_SetWatchdogTime(NTAG_HANDLE_T ntag, uint16_t time);

/**
 * \brief get the value of the watchdog timer
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	time	Watchdog time
 * \return			true on failure*
 */
BOOL NTAG_GetWatchdogTime(NTAG_HANDLE_T ntag, uint16_t *time);

#endif /* _NTAG_BRIDGE_H_ */
