/*
 * Copyright (c) 2016, NXP Semiconductor
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#ifndef _NTAG_DRIVER_H_
#define _NTAG_DRIVER_H_
/** @file ntag_driver.h
 * \brief Public interface to access a NTAG I2C tag over I2C.
 */

/** \defgroup NTAG_DRIVER
*
* \brief This component implements API responsible for Bridging the transaction between NTAG and Microcontroller
* @{
*/

/***********************************************************************/
/* INCLUDES                                                            */
/***********************************************************************/
#include "global_types.h"
#include "ntag_defines.h"
#include "stdbool.h"
#include "phDriver.h"
#include "phbalReg.h"
#include "phOsal.h"
/***********************************************************************/
/* DEFINES                                                             */
/***********************************************************************/

/***********************************************************************/
/* TYPES                                                               */
/***********************************************************************/

/**
 *
 * \brief Callback to be called when the interrupt arrives
 * \retval  None
 */
typedef void (*ppISR_EventCallBck_t)(void);
void ISR_EventCallback(void);
typedef struct NTAG_DEVICE
{
    void * pDataParams;
    ppISR_EventCallBck_t ISR_EventCallback;
    void* isr;
    NTAG_STATUS_T status;
    uint8_t AutoStanbyEn;
    BOOL blocking;             /* Flag to indicate whether system is blocking or not */
    uint16_t BlockTime;        /* Time for which to block, if system is blocking */

} NTAG_DEVICE_t, *NTAG_HANDLE_T;


#define HAVE_NTAG_INTERRUPT
typedef enum
{
#ifdef HAVE_NTAG_INTERRUPT
    NTAG_ED_PIN_STATE_HI,
    NTAG_ED_PIN_STATE_LO,
#endif

} NTAG_ED_STATE_T;

/***********************************************************************/
/* GLOBAL VARIABLES                                                    */
/***********************************************************************/
extern phOsal_SemObj_t semObj;
/***********************************************************************/
/* GLOBAL FUNCTION PROTOTYPES                                          */
/***********************************************************************/

/**
 * \brief wait till the ED Pin has the selected state
 *
 *	This functions waits until the selected event occurs or the timeout value is
 *	reached. See NTAG_EVENT_T for possible events to be waited on.
 *	If you want to use the Interrupted Events set the FD function accordingly
 *	before calling this function. Notice that some Interrupted Events are
 *	indistinguishable, so it will trigger at a false event.
 *
 * \param	ntag      	handle to identify the NTAG device instance
 * \param	event     	state to be waited for \ref NTAG_ED_STATE_T
 * \param	timeout_ms	timeout value in ms
 *
 * \return			true on timeout
 */
BOOL NTAG_WaitForEDPinState(NTAG_HANDLE_T ntag, NTAG_ED_STATE_T state, uint32_t timeout_ms);


/**
 * \brief initialize the selected NTAG device for operation
 *
 *  This function registers the specified I2C bus and returns a
 *  handle to the selected device to be used for further operations.
 *
 * \param   ntag   \ref NTAG_HANDLE_T to be used henceforth
 * \param   BAL    \ref phbalReg_Type_t the bal type intialized in bal init.
 *
 * \return          true on failure
 */
BOOL NTAG_InitDevice(NTAG_HANDLE_T ntag, void* BAL);

/**
 * \brief Configures the middleware for blocking mode
 *
 *  This function configures the middleware to Blocking/non blocking mode
 *  When selected for blocking, the middleware api gets blocked until api is
 *  successful or the blocking time is over
 *
 * \param   ntag   \ref NTAG_HANDLE_T to be used henceforth
 * \param   blocking  BOOL value 1, makes the system blocking, 0 makes non-blocking
 * \param   BlockTime Blocktime in milliseconds, blocktime 0 means it will block forever till it passes
 *
 * \return          true on failure
 */
BOOL NTAG_SetBlocking(NTAG_HANDLE_T ntag, BOOL blocking, uint16_t BlockTime);

/**
 * \brief read len number of bytes from the selected NTAG device
 *
 *	This functions reads the specified number of bytes from the selected
 *	NTAG device, starting at the given address. Reading from an invalid address
 *	will result in a failed read.
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param   address 16-bit address in device memory space to start reading from
 * \param	bytes	array of bytes to store read data
 * \param	len 	number of bytes to be read
 *
 * \return			true on failure
 */
BOOL NTAG_ReadBytes(NTAG_HANDLE_T ntag, uint16_t address, uint8_t *bytes, uint16_t len );

/**
 * \brief write len number of bytes to the selected NTAG device
 *
 *	This functions writes the specified number of bytes to the given address of
 *	the selected NTAG device. Writing to an invalid address
 *	is undefined.
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param   address byte address in device memory space to start writing to
 * \param	bytes	array of bytes to be written
 * \param	len 	number of bytes to be written
 *
 * \return			true on failure
 */
BOOL NTAG_WriteBytes(NTAG_HANDLE_T ntag, uint16_t address, const uint8_t *bytes, uint16_t len);

/**
 * \brief read a session register from the selected NTAG device
 *
 *	This functions reads the specified 16-bit session - register from the selected
 *	NTAG device. The returned value for Register of 8 bit width will have only
 *	relevent LSB
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	reg 	register offset from the start of the register block
 * \param	val 	uint to store read value
 *
 * \return			true on failure
 */
BOOL NTAG_ReadRegister(NTAG_HANDLE_T ntag, uint8_t reg, uint16_t *val);

/**
 * \brief write a session register of the selected NTAG device
 *
 *	This functions writes the specified 16-bit session - register of the selected
 *	NTAG device. For 8-bit width Register the LSB 8-bit will only be writeen
 *	Only the mask selected bits will be written.
 *
 * \param	ntag	handle to identify the NTAG device instance
 * \param	reg 	register offset from the start of the register block
 * \param	mask	only bits set to one will be written
 * \param	val 	16-bit value to be written
 *
 * \return			true on failure
 */
BOOL NTAG_WriteRegister(NTAG_HANDLE_T ntag, uint8_t reg, uint16_t mask, uint16_t val);

/**
 * \brief get the error code of the last failure
 *
 *	If the previous call to any of the read/write function failed this function
 *	will return the reason via the corresponding error code. The error code is not
 *	latched, therefore any successful read/write after a failure will reset the
 *	error code.
 *	This function will return NTAG_CLOSED on a closed handle.
 *
 * \param	ntag	handle to identify the NTAG device instance
 *
 * \return			NTAG status code
 */
NTAG_STATUS_T NTAG_GetLastError(NTAG_HANDLE_T ntag);

#endif /* _NTAG_DRIVER_H_ */
