/*
 * Copyright (c) 2016, NXP Semiconductor
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/***********************************************************************/
/* INCLUDES                                                            */
/***********************************************************************/

#include "ntag_driver_intern.h"
#include "ntag_defines.h"
#include "global_types.h"
#include "phDriver.h"
#include "phDriver_Timer.h"
#include "ntag_driver.h"
#include "phOsal.h"
#include "BoardSelection.h"
#include "phDriver_Gpio.h"
#include "ntag_bridge.h"
#include "isr_driver_intern.h"
/***********************************************************************/
/* DEFINES                                                             */
/***********************************************************************/
phOsal_SemObj_t semObj;
static const uint8_t sema[] = "sema";
void phDriver_PitTimerCallBack(void);
/***********************************************************************/
/* GLOBAL VARIABLES                                                    */
/***********************************************************************/
uint8_t Unblock;
/***********************************************************************/
/* GLOBAL PUBLIC FUNCTIONS                                             */
/***********************************************************************/


BOOL NTAG_InitDevice(NTAG_HANDLE_T ntag, void* BAL)
{
    BOOL status;
    uint16_t val;
    if((ntag == NULL) || (BAL == NULL))
    {
        return true;
    }

    ntag->pDataParams = BAL;
    ntag->ISR_EventCallback = &ISR_EventCallback;
    ntag->blocking = 0;
    ntag->BlockTime = 10;
    Unblock = 0;

    semObj.pSemName = (uint8_t*)sema;
    semObj.semInitialCount = 0;
    semObj.semMaxCount = 1;
    (void)phOsal_SemCreate(&semObj.SemHandle, &semObj,E_OS_SEM_OPT_SIGNALLING_SEM);
    /* Reading  t
     * he TAG Status Register to Check VCC OK status */
    status = NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_TAG_STATUS_REG, &val);

    /* In case of failure wait for 750 to ensure standby dint cause the problem */
    if(status == true)
    {
        phDriver_TimerStart(PH_DRIVER_TIMER_MICRO_SECS, 750, NULL);
        status = NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_TAG_STATUS_REG, &val);
    }
    /* If VCC is enabled proceed further */
    if((status == false)  && (val & 0x02))
    {
        /* Check for I2C silence */
        status = NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_I2C_SLAVE_CONF_REG, &val);
        if(val & 0x100)
        {
            status = true;
        }
        else
        {
            status = false;
        }
    }
    return status;
}

void NTAG_CloseDevice(NTAG_DEVICE_t* ntag)
{
    if(ntag->pDataParams != NULL)
    {
        ((NTAG_DEVICE_t *)(ntag))->pDataParams = NULL;
    }
}

BOOL NTAG_SetBlocking(NTAG_HANDLE_T ntag, BOOL blocking, uint16_t BlockTime)
{
    if(ntag == NULL)
    {
        return true;
    }
    ntag->blocking = blocking;
    ntag->BlockTime = BlockTime;
    return false;
}


BOOL NTAG_ReadBytes(NTAG_HANDLE_T ntag, uint16_t address, uint8_t *bytes, uint16_t len)
{
    uint16_t wRxLength = 0;
    uint8_t data[4] = {0x00, 0x00, 0x00, 0x00};
    BOOL on = 0;
    BOOL status = false;

    /* Length to be read can not be zero and Ntag cant be NULL */
    if((ntag == NULL) ||(len == 0))
    {
        return true;
    }

    /* Checking the validity of the address */
    if((address > NTAG_MEM_BLOCK_END_SRAM) || ((address >= NTAG_MEM_SESSION_START) && (address < NTAG_MEM_BLOCK_START_SRAM)) || ((address > NTAG_MEM_EEPROM_END) && (address < NTAG_MEM_CONFIGURATION_START)))
    {
        return true;
    }

    /* Assign the correct address */
    data[0] = (uint8_t)(address >> 8);
    data[1] = (uint8_t)(address);

    /* If Passthrough mode transaction can never be blocking, even if the get fails we proceed */
    if(!(NTAG_GetPthruOnOff( ntag, &on)))
    {
        if(!on)
        {
            ntag->blocking = 0;
        }
    }

    /* Start the timer for which to block */
    if((ntag->blocking) && (!ntag->BlockTime))
    {
        (void)phDriver_TimerStart(PH_DRIVER_TIMER_MILLI_SECS, ntag->BlockTime, &phDriver_PitTimerCallBack);
    }

    /* Length for SRAM can't be more than 256 */
    if((address > NTAG_MEM_BLOCK_START_SRAM) && (len > 256))
    {
        return true;
    }

    /* If the user wants the api to be blocking and timer has not expired */
    do
    {
        /* Read the addressed byte */
        status = phbalReg_Exchange(((NTAG_DEVICE_t *)(ntag))->pDataParams,  0, &data[0], 2, len,  bytes, &wRxLength);
    }while((ntag->blocking && (!Unblock)) && status);

    if(status)
    {
        ntag->status = NTAG_ERROR_RX_FAILED;
        return true;
    }

    if (wRxLength != len)
    {
        ntag->status = NTAG_ERROR_RX_FAILED;
        return true;
    }

    return false;
}

BOOL NTAG_WriteBytes(NTAG_HANDLE_T ntag, uint16_t address, const uint8_t *bytes, uint16_t len)
{
    uint16_t bytes_written = 0;
    uint16_t wRxLength = 0;
    uint8_t data[258] = {0x00};
    uint32_t count = 0;
    uint16_t current_len = 0;
    uint32_t i = 0;
    BOOL on = 0;

    /* Length to be written can not be zero */
    if(len == 0)
    {
        return true;
    }

    current_len = len;

    /* If Passthrough mode transaction can never be blocking */
    if(!(NTAG_GetPthruOnOff( ntag, &on)))
    {
        if(!on)
        {
            ntag->blocking = 0;
        }
    }
    if((ntag->blocking) && (!ntag->BlockTime))
    {
        (void)phDriver_TimerStart(PH_DRIVER_TIMER_MILLI_SECS, ntag->BlockTime, &phDriver_PitTimerCallBack);
    }

    if(address < NTAG_MEM_BLOCK_START_SRAM)
    {
        while (bytes_written < len)
        {
            if (current_len < NTAG_I2C_BLOCK_SIZE)
            {
                /* read block into ntag->rx_buffer only */
                if (NTAG_ReadBlock(ntag, address + count, &data[0], NTAG_I2C_BLOCK_SIZE))
                {
                    return true;
                }


                for(i = 0; i < current_len; i++)
                {
                    data[i] =  *(bytes + bytes_written);
                    bytes_written++;
                }


                /* read block into ntag->rx_buffer only */
                if (NTAG_WriteBlock(ntag, address + count, &data[0], NTAG_I2C_BLOCK_SIZE))
                    return true;

                /* modify rx_buffer */

                bytes_written += current_len;
            }
            else
            {
                /* full block read */
                if (NTAG_WriteBlock(ntag, address + count, bytes + bytes_written, NTAG_I2C_BLOCK_SIZE))
                    return true;
                current_len -= NTAG_I2C_BLOCK_SIZE;
            }
            count++;
            bytes_written += NTAG_I2C_BLOCK_SIZE;
        }
    }
    else
    {
        /* Length for SRAM can't be more than 256 */
        if(len > 256)
        {
            return true;
        }

        data[0] = (uint8_t)(address >> 8);
        data[1] = (uint8_t)(address);

        memcpy(&data[2], bytes, len );
        /* Read the Register byte */
        if (HAL_I2C_OK != phbalReg_Exchange(ntag->pDataParams,  0, &data[0], len + 2, 0, NULL, &wRxLength))
        {
            return true;
        }
    }

    return false;
}

BOOL NTAG_ReadRegister(NTAG_HANDLE_T ntag, uint8_t reg, uint16_t *val)
{
    uint8_t data[4] = {0x00};
    uint8_t Rxdata[2] ={0x00};
    uint16_t wRxLength;
    if(ntag == NULL)
    {
        return true;
    }
    data[0] = (uint8_t)(SessionRegister[reg][0] >> 8);
    data[1] = (uint8_t)(SessionRegister[reg][0]);
    data[2] = (uint8_t)(SessionRegister[reg][1]);

    /* Read the Register byte */
    if (HAL_I2C_OK != phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 3, 1,  &Rxdata[0], &wRxLength))
    {
        if(ntag->AutoStanbyEn)
        {
            (void)phDriver_TimerStart(PH_DRIVER_TIMER_MICRO_SECS, 500, NULL);
            ntag->AutoStanbyEn = 0;
            if(HAL_I2C_OK !=  phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 6, 0,  NULL, &wRxLength))
            {
                ntag->status = NTAG_ERROR_TX_FAILED;
                return true;
            }
        }

        ntag->status = NTAG_ERROR_TX_FAILED;
        return true;

    }

    if(SessionRegister[reg][2] > 1)
    {
        data[2] = (uint8_t)(SessionRegister[reg][1] + 1);
        /* Read the Register byte */
        if (HAL_I2C_OK != phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 3, 1,  &Rxdata[1], &wRxLength))
        {
            ntag->status = NTAG_ERROR_TX_FAILED;
            return true;
        }
        *val = (uint16_t)((uint16_t)(Rxdata[1] << 8) | Rxdata[0]);
    }
    else
    {
        *val = Rxdata[0];
    }
    return false;
}

BOOL NTAG_WriteRegister(NTAG_HANDLE_T ntag, uint8_t reg, uint16_t mask, uint16_t val)
{
    uint8_t data[5] = {0x00};
    uint16_t wRxLength;
    if(ntag == NULL)
    {
        return true;
    }
    data[0] = (uint8_t)(SessionRegister[reg][0] >> 8);
    data[1] = (uint8_t)(SessionRegister[reg][0]);
    data[2] = (uint8_t)(SessionRegister[reg][1]);
    data[3] = (uint8_t)(mask);
    data[4] = (uint8_t)(val);

    if (HAL_I2C_OK != phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 5, 0,  NULL, &wRxLength))
    {
        if(ntag->AutoStanbyEn)
        {
            (void)phDriver_TimerStart(PH_DRIVER_TIMER_MICRO_SECS, 500, NULL);
            ntag->AutoStanbyEn = 0;
            if(HAL_I2C_OK != phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 5, 0,  NULL, &wRxLength))
            {
                ntag->status = NTAG_ERROR_TX_FAILED;
                return true;
            }
        }
        ntag->status = NTAG_ERROR_TX_FAILED;
        return true;

    }

    if(SessionRegister[reg][2] > 1)
    {
        data[2] = (uint8_t)(SessionRegister[reg][1] + 1);
        data[3] = (uint8_t)(mask >> 8);
        data[4] = (uint8_t)(val >> 8);

        /* Read the Register byte */
        if (HAL_I2C_OK != phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 5, 0,  NULL, &wRxLength))
        {
            ntag->status = NTAG_ERROR_TX_FAILED;
            return true;
        }
    }

    return false;
}

NTAG_STATUS_T NTAG_GetLastError(NTAG_HANDLE_T ntag)
{
    return ntag->status;
}

/***********************************************************************/
/* GLOBAL PRIVATE FUNCTIONS                                            */
/***********************************************************************/
//#ifdef CTC
//#pragma CTC SKIP
BOOL NTAG_ReadBlock(NTAG_HANDLE_T ntag, uint16_t block, uint8_t *bytes, uint8_t len)
{
    uint32_t i = 0;
    uint16_t status = PH_DRIVER_FAILURE;
    uint16_t wRxLength = 0;
    uint8_t data[4] = {0x00, 0x00, 0x00, 0x00};
    data[0] = (uint8_t)(block >> 8);
    data[1] = (uint8_t)(block);

    /* send block number */
    if (HAL_I2C_OK != phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 2, 0,  NULL, &wRxLength))
    {
        /* If it failed due to standby, try again after 500us */
        if(ntag->AutoStanbyEn)
        {
            (void)phDriver_TimerStart(PH_DRIVER_TIMER_MICRO_SECS, 500, NULL);
            ntag->AutoStanbyEn = 0;
            status = phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 2, 0,  NULL, &wRxLength);
        }

        /* If transaction failed and blocking retry */
        if(status && ntag->blocking)
        {
            /* Set the Block timer */
            if(ntag->BlockTime)
            {
                (void)phDriver_TimerStart(PH_DRIVER_TIMER_MILLI_SECS, ntag->BlockTime, &phDriver_PitTimerCallBack);
            }

            /* If if the transaction failed and mode is blocking, keep on trying till block time is over */
            while(status && (!Unblock))
            {
                 status = phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 2, 0,  NULL, &wRxLength);
                 if(!status)
                 {
                     break;
                 }
            }
        }
        if(status != HAL_I2C_OK)
        {
            ntag->status = NTAG_ERROR_TX_FAILED;
            return true;
        }
    }

    /* receive bytes */
    if (HAL_I2C_OK != phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 0, 4,  &data[0], &wRxLength))
    {
        /* If if the transaction failed and mode is blocking, keep on trying till block time is over */
        while(status && (!Unblock))
        {
             status = phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 2, 0,  NULL, &wRxLength);
             if(!status)
             {
                 break;
             }
        }
        if(status != HAL_I2C_OK)
        {
            ntag->status = NTAG_ERROR_RX_FAILED;
            return true;
        }
    }

    len = MIN(len, NTAG_I2C_BLOCK_SIZE);

    /* write to bytes buffer */
    for (i = 0; i < len; i++)
        bytes[i] = data[i];

    return false;
}
//#pragma CTC ENDSKIP
//#endif
//#ifdef CTC
//#pragma CTC SKIP
BOOL NTAG_WriteBlock(NTAG_HANDLE_T ntag, uint16_t block, const uint8_t *bytes, uint8_t len)
{
    uint16_t Tag_Status_reg = 0;
    uint16_t status= PH_DRIVER_FAILURE;
    uint32_t timeout = NTAG_MAX_WRITE_DELAY_MS / 5 + 1;
    uint32_t i = 0;
    uint8_t data[6] = {0x00, 0x00, 0x00, 0x00};
    uint16_t wRxLength = 0;

    if(ntag == NULL)
    {
        return true;
    }

    /* MSB to be addressed first */
    data[0] = (uint8_t)(block >> 8);
    data[1] = (uint8_t)(block);

    len = MIN(len, NTAG_I2C_BLOCK_SIZE);

    /* copy len bytes */
    for (i = 0; i < len; i++)
        data[i + 2] = bytes[i];

    /* Clearing the eeprom write error that could have been due to earlier RF Transaction */
    (void)NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_TAG_STATUS_REG, NTAG_MEM_TAG_STATUS_MASK_EEPROM_WR_ERR, 0);

    /* send block address and data */
    if (HAL_I2C_OK != phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 6, 0,  NULL, &wRxLength))
    {
        /* If it failed due to standby, try again after 500us */
        if(ntag->AutoStanbyEn)
        {
            (void)phDriver_TimerStart(PH_DRIVER_TIMER_MICRO_SECS, 500, NULL);
            ntag->AutoStanbyEn = 0;
            status = phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 6, 0,  NULL, &wRxLength);
        }

        /* If transaction failed and blocking retry */
        if(status && ntag->blocking)
        {
            /* Set the Block timer */
            if(ntag->BlockTime)
            {
                (void)phDriver_TimerStart(PH_DRIVER_TIMER_MILLI_SECS, ntag->BlockTime, &phDriver_PitTimerCallBack);
            }

            /* If if the transaction failed and mode is blocking, keep on trying till block time is over */
            while(status && (!Unblock))
            {
                 status = phbalReg_Exchange(ntag->pDataParams,  0, &data[0], 6, 0,  NULL, &wRxLength);
                 if(!status)
                 {
                     break;
                 }
            }
        }
        if(status != HAL_I2C_OK)
        {
            ntag->status = NTAG_ERROR_TX_FAILED;
            return true;
        }
    }

    /* do not wait for completion when writing SRAM */
    if (block >= NTAG_MEM_BLOCK_START_SRAM && block < (NTAG_MEM_BLOCK_START_SRAM + NTAG_MEM_SRAM_BLOCKS))
        return true;

    /* wait for completion */
    do
    {
        phDriver_TimerStart(PH_DRIVER_TIMER_MILLI_SECS, 5, NULL);
        if (NTAG_ReadRegister(ntag, NTAG_MEM_OFFSET_TAG_STATUS_REG, &Tag_Status_reg))
            break;
        timeout--;
    } while (timeout && (Tag_Status_reg & NTAG_MEM_TAG_STATUS_MASK_EEPROM_WR_BUSY));

    if (0 == timeout)
    {
        ntag->status = NTAG_ERROR_WRITE_TIMEOUT;
        return true;
    }
    if(Tag_Status_reg & NTAG_MEM_TAG_STATUS_MASK_EEPROM_WR_ERR)
    {
        NTAG_WriteRegister(ntag, NTAG_MEM_OFFSET_TAG_STATUS_REG, NTAG_MEM_TAG_STATUS_MASK_EEPROM_WR_ERR, 0);
        ntag->status = NTAG_EEPROM_WRITE_ERROR;
        return true;
    }

    return false;
}
//#pragma CTC ENDSKIP
//#endif
/**
 * \brief wait till the ED Pin has the selected state
 *
 *  This functions waits until the selected event occurs or the timeout value is
 *  reached. See NTAG_EVENT_T for possible events to be waited on.
 *  If you want to use the Interrupted Events set the FD function accordingly
 *  before calling this function. Notice that some Interrupted Events are
 *  indistinguishable, so it will trigger at a false event.
 *
 * \param   ntag        handle to identify the NTAG device instance
 * \param   event       state to be waited for
 * \param   timeout_ms  timeout value in ms
 *
 * \return          true on timeout
 */
BOOL NTAG_WaitForEDPinState(NTAG_HANDLE_T ntag, NTAG_ED_STATE_T state, uint32_t timeout_ms)
{
    uint16_t status = 0;
    phOsal_TimerPeriodObj_t timePeriodToWait;
    phOsal_TimerPeriodObj_t semClearTime;
    semClearTime.unitPeriod = OS_TIMER_UNIT_MSEC;
    semClearTime.period = 0;
    timePeriodToWait.unitPeriod = OS_TIMER_UNIT_MSEC;
    timePeriodToWait.period = timeout_ms;

    if(ntag == NULL)
    {
        return true;
    }

    /* Clearing any pending semaphore */
    (void)phOsal_SemPend(semObj.SemHandle, semClearTime);

    switch (state)
    {
        case NTAG_ED_PIN_STATE_HI:
            HAL_ISR_RegisterCallback(PHDRIVER_PIN_IRQ, PH_DRIVER_INTERRUPT_LEVELONE);
            status = phOsal_SemPend(semObj.SemHandle, timePeriodToWait);
            if(status)
                return true;

            break;
        case NTAG_ED_PIN_STATE_LO:
            HAL_ISR_RegisterCallback(PHDRIVER_PIN_IRQ, PH_DRIVER_INTERRUPT_LEVELZERO);
            status = phOsal_SemPend(semObj.SemHandle, timePeriodToWait);
            if(status)
                return true;
            break;

        default:
            break;
    }
    return false;
}

void ISR_EventCallback(void)
{
    phOsal_SemPost(semObj.SemHandle, E_OS_SEM_OPT_SIGNALLING_SEM);
}


void phDriver_PitTimerCallBack(void)
{
    Unblock = 1;
}
