/*
*         Copyright (c), NXP Semiconductors Bangalore / India
*
*                     (C)NXP Semiconductors
*       All rights are reserved. Reproduction in whole or in part is
*      prohibited without the written consent of the copyright owner.
*  NXP reserves the right to make changes without notice at any time.
* NXP makes no warranty, expressed, implied or statutory, including but
* not limited to any implied warranty of merchantability or fitness for any
*particular purpose, or that the use will not infringe any third party patent,
* copyright or trademark. NXP must not be liable for any loss or damage
*                          arising from its use.
*/

/** \file
* Timer of MicroController Platform.
* $Author$
* $Revision$
* $Date$
*
*/

#ifndef PHPLATFORM_H
#define PHPLATFORM_H

#define NXPBUILD__PH_LPC1769
#define NXPBUILD__PH_PLATFORM

#ifdef  __cplusplus
extern "C" {
#endif    /* __cplusplus */

#ifdef NXPBUILD__PH_PLATFORM
#include <phOsal.h>

typedef uint16_t phStatus_t;
/** \defgroup ph_Error Error Code Collection
* \brief Common Error Codes
* @{
*/

/** \name Success Codes
*/
/*@{*/
#define PH_ERR_SUCCESS                  0x0000U /**< Returned in case of no error. */
#define PH_ERR_RESOURCE_ERROR           0x000DU /**< An resource error. */
#define PH_ERR_PARAMETER_OVERFLOW       0x0022U /**< Reading/Writing a parameter would produce an overflow. */


/** \defgroup phPlatform Platform Component
* \brief These Components implement the Platform abstraction layer.
* @{
*/

/* ------------------------------------------------ */
/* General port definition                          */
/* ------------------------------------------------ */
#define PH_PLATFORM_MAX_TIMERS          4U         /**< Maximum number of timers available. */

/**
* \name Timer Configuration
* @{ */
#define PH_PLATFORM_TIMER_SINGLE_SHOT   0x0000U    /**< On Timer expire it will be Stopped and Timercallback will be called. */
#define PH_PLATFORM_TIMER_FREE_RUN      0x0001U    /**< Auto reloads on expire of the Timer and Timercallback will be called. */
/* @} */

/**
* \name Timer Units
* @{ */
#define PH_PLATFORM_TIMER_UNIT_US       1000000U   /**< Indicates that the specified delay is in microseconds.*/
#define PH_PLATFORM_TIMER_UNIT_MS       1000U      /**< Indicates that the specified delay is in milliseconds. */
#define PH_PLATFORM_TIMER_UNIT_S        1U         /**< Indicates that the specified delay is in seconds. */
/* @} */

/**
*
* \brief Tick Timer ISR callback which will be called by ISR handler implementation in Platform Layer.
* \retval  None
*/
typedef void (*pphPlatform_TickTimerISRCallBck_t)(void);

/**
*
* \brief Timer callback interface which will be called once registered timer
* timeout expires.
* \param[in] TimerId  Timer Id for which callback is called.
* \retval  None
*/
typedef void (*pphPlatform_CallBck_t)(uint8_t bTimerId);

/**
 * Status of the timer.
 */
typedef enum phPlatform_Timer_Status
{
    E_TIMER_FREE = 0,//!< E_TIMER_FREE
    E_TIMER_ACTIVE,  //!< E_TIMER_ACTIVE
    E_TIMER_RUNNING, //!< E_TIMER_RUNNING
    E_TIMER_STOPPED  //!< E_TIMER_STOPPED
} phPlatform_Timer_Status_t;

typedef struct phPlatform_Timer_Struct
{
    void                     *pHwTimerHandle;                        /**< Timer handle for the requested hardware timer. */
    pphPlatform_CallBck_t     pTimerCallback;                        /**< The call back function for this timer. */
    uint32_t                  dwUnits;                               /**< Unit configured by user. */
    uint8_t                   bTimerId;                              /**< ID of the timer. */
    phPlatform_Timer_Status_t bTimerStatus;                          /**< Indicates current timer status. */
} phPlatform_Timer_Struct_t;

/**
* \brief OSAL parameter structure
*/
typedef struct
{
    phPlatform_Timer_Struct_t           gTimers[PH_PLATFORM_MAX_TIMERS]; /**< Timer structure. */
} phPlatform_DataParams_t;

extern phPlatform_DataParams_t *gpphPlatform_DataParams;

/**
 * \brief Initializes the Timer component.
 * @note This Api must be called before calling any other Timer Api.
 *
 * @param[in]  pDataParams   Pointer to this layers parameter structure.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 */
phStatus_t phPlatform_Timer_Init(phPlatform_DataParams_t *pDataParams);

/**
 * \brief Allocates a free timer.
 *
 * @param[in]  pDataParams   Pointer to this layers parameter structure.
 * @param[in]  dwUnits       Timer tick units.
 * @param[out] ppTimerHandle Timer handle in case of success else NULL.
 *
 * @return void
 */
void phPlatform_Timer_Create(phPlatform_DataParams_t    *pDataParams,
                             uint32_t                    dwUnits,
                             phPlatform_Timer_Struct_t **ppTimerHandle
                             );

/**
 * \brief This API configures the acquired timer with timeout duration and callback to be called upon timer expire.
 *
 * @param[in] pTimerHandle  Timer handle returned by \ref phPlatform_Timer_Create.
 * @param[in] dwTimePeriod  Time delay count.
 * @param[in] pCallBackFunc Callback to be called on time out.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 * @retval #PH_ERR_PARAMETER_OVERFLOW If timer cannot be configured to the provide time by parameter dwTimePeriod.
 */
phStatus_t phPlatform_Timer_Configure(phPlatform_Timer_Struct_t       *pTimerHandle,
                                      uint32_t                         dwTimePeriod,
                                      pphPlatform_CallBck_t            pCallBackFunc
                                      );

/**
 * \brief Start the timer.
 *
 * @param[in] pTimerHandle Timer handle returned by \ref phPlatform_Timer_Create.
 * @param[in] wOption Parameter to mention Single shot \ref PH_PLATFORM_TIMER_SINGLE_SHOT
 *                    or Free running timer \ref PH_PLATFORM_TIMER_FREE_RUN.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 * @retval #PH_ERR_RESOURCE_ERROR Timer could not be started.
 */
phStatus_t phPlatform_Timer_Start(phPlatform_Timer_Struct_t *pTimerHandle,
                                  uint16_t                   wOption
                                  );

/**
 * \brief Get Elapsed Delay from timer which is created and started before calling this API.
 *
 * @param[in]  pTimerHandle      Timer handle returned by \ref phPlatform_Timer_Create.
 * @param[out] dwGetElapsedDelay Measured Delay after Timer Start in sec/milli/micro seconds based on input option.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 */
phStatus_t phPlatform_Timer_GetElapsedDelay(phPlatform_Timer_Struct_t *pTimerHandle,
                                            uint32_t                  *dwGetElapsedDelay
                                            );

/**
 * \brief Stop the timer.
 *
 * @param[in] pTimerHandle Timer handle returned by \ref phPlatform_Timer_Create.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 * @retval #PH_ERR_RESOURCE_ERROR Timer could not be stopped.
 */
phStatus_t phPlatform_Timer_Stop(phPlatform_Timer_Struct_t *pTimerHandle);

/**
 * \brief Resets the timer allocated to the application which is created by \ref phPlatform_Timer_Create.
 *
 * @param[in] pTimerHandle Timer handle returned by \ref phPlatform_Timer_Create.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 * @retval #PH_ERR_RESOURCE_ERROR Timer could not be stopped.
 */
phStatus_t phPlatform_Timer_Reset(phPlatform_Timer_Struct_t *pTimerHandle);

/**
 * \brief Delete or Deallocate the timer.
 *
 * @param[in] pTimerHandle Timer handle returned by \ref phPlatform_Timer_Create.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 * @retval #PH_ERR_RESOURCE_ERROR Timer could not be stopped.
 */
phStatus_t phPlatform_Timer_Delete(phPlatform_Timer_Struct_t *pTimerHandle);

/**
 * \brief DeInitializes the Timer component.
 *
 * @param[in]  pDataParams   Pointer to this layers parameter structure.
 *
 * @return void
 */
void phPlatform_Timer_DeInit(phPlatform_DataParams_t *pDataParams);


#include <phPlatform_Port_Host.h>

#if defined (NXPBUILD__PH_LPC1769) || defined (NXPBUILD__PH_LPC11U68) || defined (NXPBUILD__PH_PN7462AU) || defined (NXPBUILD__PH_KINETIS)
#  include <phPlatform_Port_Cortex.h>
#endif

#endif /* NXPBUILD__PH_PLATFORM */

#ifdef  __cplusplus
}
#endif /* __cplusplus */

#endif /* PHPLATFORM_H */
