/*
*         Copyright (c), NXP Semiconductors Bangalore / India
*
*                     (C)NXP Semiconductors
*       All rights are reserved. Reproduction in whole or in part is
*      prohibited without the written consent of the copyright owner.
*  NXP reserves the right to make changes without notice at any time.
* NXP makes no warranty, expressed, implied or statutory, including but
* not limited to any implied warranty of merchantability or fitness for any
*particular purpose, or that the use will not infringe any third party patent,
* copyright or trademark. NXP must not be liable for any loss or damage
*                          arising from its use.
*/

/** \file
* Platform file abstracting functions of Host controller.
* $Author$
* $Revision$
* $Date$
*
*/

#ifndef PHPLATFORM_PORT_HOST_H
#define PHPLATFORM_PORT_HOST_H


/**
 * \brief Initialize the Host Timer module.
 * @note This API will be called by \ref phPlatform_Timer_Init API.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INTERNAL_ERROR For unsuccessful operation.
 */
phStatus_t phPlatform_Port_Host_HwTimer_Init(void);

/**
 * \brief Allocates a free Host timer.
 * @note This API will be called by \ref phPlatform_Timer_Create API.
 *
 * @param[in] dwUnits Timer Resolution (e.g. \ref PH_PLATFORM_TIMER_UNIT_S)
 * @param[in] bTimerId  Timer Number
 * @param[out] ppTimerHandle Handle to the timer in case of success else NULL.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_RESOURCE_ERROR If host timer cannot be created.
 */
phStatus_t phPlatform_Port_Host_HwTimer_Create(uint32_t dwUnits, uint8_t bTimerId, void** ppTimerHandle);

/**
 * \brief This API configures the acquired timer with timeout duration.
 * @note This API will be called by \ref phPlatform_Timer_Configure API.
 *
 * @param[in] pTimerHandle Handle to timer as returned by \ref phPlatform_Port_Host_HwTimer_Create.
 * @param[in] dwUnits Timer Resolution configured during \ref phPlatform_Port_Host_HwTimer_Create.
 * @param[in] dwTimePeriod
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 * @retval #PH_ERR_PARAMETER_OVERFLOW If timer cannot be configured to the provide time by parameter dwTimePeriod.
 */
phStatus_t phPlatform_Port_Host_HwTimer_Configure(void *pTimerHandle, uint32_t dwUnits, uint32_t dwTimePeriod);

/**
 * \brief Start the Host timer, upon timer expire \ref phPlatform_Timer_IsrHandler API will be called with timer ID.
 * @note This API will be called by \ref phPlatform_Timer_Start API.
 *
 * @param[in] pTimerHandle Handle to timer as returned by \ref phPlatform_Port_Host_HwTimer_Create.
 * @param[in] wOption Parameter to mention Single shot \ref PH_PLATFORM_TIMER_SINGLE_SHOT
 *                    or Free running timer \ref PH_PLATFORM_TIMER_FREE_RUN.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 * @retval #PH_ERR_RESOURCE_ERROR If host timer could not be started.
 */
phStatus_t phPlatform_Port_Host_HwTimer_Start(void * pTimerHandle, uint16_t wOption);

/**
 * \brief Get Elapsed Delay from host timer which is created and started before calling this API.
 * @note This API will be called by \ref phPlatform_Timer_GetElapsedDelay API.
 *
 * @param[in] pTimerHandle Handle to timer as returned by \ref phPlatform_Port_Host_HwTimer_Create.
 * @param[in] dwUnits Timer Resolution configured during \ref phPlatform_Port_Host_HwTimer_Create.
 * @param[out] pdwGetElapsedDelay Duration of delay as per the passed resolution.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 */
phStatus_t phPlatform_Port_Host_HwTimer_GetElapsedDelay(void * pTimerHandle, uint32_t dwUnits, uint32_t *pdwGetElapsedDelay);

/**
 * \brief Stop the host timer.
 * @note This API will be called by \ref phPlatform_Timer_Stop API.
 *
 * @param[in] pTimerHandle Handle to timer as returned by \ref phPlatform_Port_Host_HwTimer_Create.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 * @retval #PH_ERR_RESOURCE_ERROR Timer could not be stopped.
 */
phStatus_t phPlatform_Port_Host_HwTimer_Stop(void * pTimerHandle);

/**
 * \brief Reset the host timer.
 * @note This API will be called by \ref phPlatform_Timer_Reset API.
 *
 * @param[in] pTimerHandle Handle to timer as returned by \ref phPlatform_Port_Host_HwTimer_Create.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 * @retval #PH_ERR_RESOURCE_ERROR Could not reset timer.
 */
phStatus_t phPlatform_Port_Host_HwTimer_Reset(void * pTimerHandle);

/**
 * \brief Delete the host timer.
 * @note This API will be called by \ref phPlatform_Timer_Delete API.
 *
 * @param[in] pTimerHandle Handle to timer as returned by \ref phPlatform_Port_Host_HwTimer_Create
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 * @retval #PH_ERR_INVALID_PARAMETER Invalid input parameter.
 * @retval #PH_ERR_RESOURCE_ERROR Timer could not delete.
 */
phStatus_t phPlatform_Port_Host_HwTimer_Delete(void * pTimerHandle);

/**
 * \brief DeInitializes the Timer component.
 * @note This API will be called by \ref phPlatform_Timer_DeInit API.
 *
 * @return Status of the API
 * @retval #PH_ERR_SUCCESS Operation successful.
 */
void phPlatform_Port_Host_HwTimer_DeInit(void);

/**
 * @}
 */

#endif /* PHPLATFORM_PORT_HOST_H */
