/*
 *         Copyright (c), NXP Semiconductors Bangalore / India
 *
 *                     (C)NXP Semiconductors
 *       All rights are reserved. Reproduction in whole or in part is
 *      prohibited without the written consent of the copyright owner.
 *  NXP reserves the right to make changes without notice at any time.
 * NXP makes no warranty, expressed, implied or statutory, including but
 * not limited to any implied warranty of merchantability or fitness for any
 *particular purpose, or that the use will not infringe any third party patent,
 * copyright or trademark. NXP must not be liable for any loss or damage
 *                          arising from its use.
 */

/** \file
 * Platform file to abstract HOST and front-end.
 * $Author$
 * $Revision$
 * $Date$
 *
 */
#include "phPlatform.h"

/*******************************************************************************
**   Function Declarations
*******************************************************************************/

/*******************************************************************************
**   Global Variable Declaration
*******************************************************************************/
/* Platform dataparams will contain registered application callback
 * that needs to be called in Interrupt handler. */
phPlatform_DataParams_t *gpphPlatform_DataParams;


phStatus_t phPlatform_Timer_Init(phPlatform_DataParams_t *pDataParams)
{
    uint8_t bCnt;

    gpphPlatform_DataParams = pDataParams;
    for(bCnt=0; bCnt < PH_PLATFORM_MAX_TIMERS; bCnt++)
    {
        pDataParams->gTimers[bCnt].pTimerCallback = NULL;
        pDataParams->gTimers[bCnt].bTimerStatus = E_TIMER_FREE;
    }

    return phPlatform_Port_Host_HwTimer_Init();
}

void phPlatform_Timer_Create(phPlatform_DataParams_t *pDataParams, uint32_t dwUnits,
        phPlatform_Timer_Struct_t **ppTimerHandle)
{
    uint8_t bCnt;
    phStatus_t wStatus;

    *ppTimerHandle = (phPlatform_Timer_Struct_t *)NULL;
    do{
        /* Check for Memory Resource to Handle Hw Timer. */
        /* shared memory, keep under critical section. */
        phPlatform_EnterCriticalSection();
        for(bCnt=0; bCnt < PH_PLATFORM_MAX_TIMERS; bCnt++)
        {
            if(pDataParams->gTimers[bCnt].bTimerStatus == E_TIMER_FREE)
            {
                pDataParams->gTimers[bCnt].bTimerStatus = E_TIMER_ACTIVE;
                pDataParams->gTimers[bCnt].bTimerId = bCnt;
                pDataParams->gTimers[bCnt].dwUnits = dwUnits;
                *ppTimerHandle  = &pDataParams->gTimers[bCnt];
                break;
            }
        }
        phPlatform_ExitCriticalSection();
        /* Insufficient Memory resource. */
        if(bCnt == PH_PLATFORM_MAX_TIMERS)
        {
            break;
        }

        /* check for hw timer availability. */
        wStatus = phPlatform_Port_Host_HwTimer_Create(dwUnits, bCnt, &pDataParams->gTimers[bCnt].pHwTimerHandle);
        if ((pDataParams->gTimers[bCnt].pHwTimerHandle == NULL) || (wStatus != PH_ERR_SUCCESS))
        {
            /* Hw Timer s not avail, release acquired Shared memory resource. */
            phPlatform_EnterCriticalSection();
            pDataParams->gTimers[bCnt].bTimerStatus = E_TIMER_FREE;
            *ppTimerHandle = (phPlatform_Timer_Struct_t *)NULL;
            phPlatform_ExitCriticalSection();
            break;
        }

    }while(0);
}

phStatus_t phPlatform_Timer_Configure(phPlatform_Timer_Struct_t *pTimerHandle, uint32_t dwTimePeriod,
        pphPlatform_CallBck_t pCallBackFunc)
{
    pTimerHandle->pTimerCallback = pCallBackFunc;
    return phPlatform_Port_Host_HwTimer_Configure(pTimerHandle->pHwTimerHandle, pTimerHandle->dwUnits,
            dwTimePeriod);
}

phStatus_t phPlatform_Timer_Start(phPlatform_Timer_Struct_t *pTimerHandle, uint16_t wOption)
{
    phStatus_t wStatus;
    wStatus = phPlatform_Port_Host_HwTimer_Start(pTimerHandle->pHwTimerHandle, wOption);

    if(wStatus == PH_ERR_SUCCESS)
    {
        pTimerHandle->bTimerStatus = E_TIMER_RUNNING;
    }

    return wStatus;
}

phStatus_t phPlatform_Timer_GetElapsedDelay(phPlatform_Timer_Struct_t *pTimerHandle, uint32_t *dwGetElapsedDelay)
{
    return phPlatform_Port_Host_HwTimer_GetElapsedDelay(pTimerHandle->pHwTimerHandle, pTimerHandle->dwUnits,
            dwGetElapsedDelay);
}

phStatus_t phPlatform_Timer_Stop(phPlatform_Timer_Struct_t *pTimerHandle)
{
    phStatus_t wStatus;
    wStatus = phPlatform_Port_Host_HwTimer_Stop(pTimerHandle->pHwTimerHandle);

    if(wStatus == PH_ERR_SUCCESS)
    {
        pTimerHandle->bTimerStatus = E_TIMER_STOPPED;
    }

    return wStatus;
}

phStatus_t phPlatform_Timer_Reset(phPlatform_Timer_Struct_t *pTimerHandle)
{
    phStatus_t wStatus;
    wStatus = phPlatform_Port_Host_HwTimer_Reset(pTimerHandle->pHwTimerHandle);

    if(wStatus == PH_ERR_SUCCESS)
    {
        pTimerHandle->bTimerStatus = E_TIMER_ACTIVE;
    }

    return wStatus;
}

phStatus_t phPlatform_Timer_Delete(phPlatform_Timer_Struct_t *pTimerHandle)
{
    phStatus_t wStatus;
    wStatus = phPlatform_Port_Host_HwTimer_Delete(pTimerHandle->pHwTimerHandle);

    if(wStatus == PH_ERR_SUCCESS)
    {
        pTimerHandle->pTimerCallback = NULL;
        pTimerHandle->bTimerStatus = E_TIMER_FREE;
    }

    return wStatus;
}

void phPlatform_Timer_DeInit(phPlatform_DataParams_t *pDataParams)
{
    uint8_t bCnt;

    for(bCnt=0; bCnt < PH_PLATFORM_MAX_TIMERS; bCnt++)
    {
        pDataParams->gTimers[bCnt].pTimerCallback = NULL;
        pDataParams->gTimers[bCnt].bTimerStatus = E_TIMER_FREE;
    }

    phPlatform_Port_Host_HwTimer_DeInit();
}

void phPlatform_Timer_IsrHandler(uint8_t bTimerId)
{
    if(gpphPlatform_DataParams->gTimers[bTimerId].pTimerCallback != NULL)
    {
        gpphPlatform_DataParams->gTimers[bTimerId].pTimerCallback(bTimerId);
    }
}

/******************************************************************************
 **                            End Of File
 ******************************************************************************/
